@@ -305,10 +305,11 @@ pfs_baz_search (struct arch_archive * a,
       file = rel_peek_str (files, x, 0);
       if (   arch_valid_package_name (file, arch_no_archive, req_type, 1)
           && !str_cmp_prefix (prefix, file))
-        rel_add_records (&answer, rel_make_record_1_taking (rel_make_field_str (arch_parse_package_name (ret_type, NULL, file))), 0);
+        rel_add_records (&answer, rel_make_record_1_taking (rel_make_field_str (arch_parse_package_name (ret_type, NULL, file))), rel_record_null);
     }
 
   rel_free_table (files);
+  rel_sort_table_by_field (0, answer, 0);
   rel_uniq_by_field (&answer, 0);
   return answer;
 }
