@@ -32,12 +32,27 @@ static void arch_print_file_metadata_diffs (int out_fd,
 static void arch_print_dir_metadata_diffs (int out_fd,
                                            struct arch_changeset_report * report,
                                            int escape_classes);
+static void print_custom_diffs(int fd, struct arch_changeset_report * report,
+                               t_uchar * orig_dir, t_uchar * mod_dir,
+                               t_uchar ** opts);
+
 static void arch_print_file_diffs (int out_fd,
                                    struct arch_changeset_report * report,
                                    int escape_classes);
 static void arch_print_added_file_diffs (int out_fd,
                                          struct arch_changeset_report * report,
                                          int escape_classes);
+static void
+print_removed_file_diffs (int fd, struct arch_changeset_report * report,
+                          t_uchar ** opts);
+static void print_file_metadata (int out_fd,
+                                 struct arch_changeset_report * report,
+                                 int escape_classes);
+static void print_dir_metadata (int out_fd,
+                                struct arch_changeset_report * report,
+                                int escape_classes);
+static void print_diffs_new(int fd, struct arch_changeset_report * report);
+
 static char * no_dot (char *name);
 static void report_unique_files_and_symlinks (rel_table * files, rel_table * symlinks,
                                               rel_table index,
@@ -295,7 +310,7 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
                              rel_make_record_3_taking (rel_make_field_str (patch_file_loc),
                                                        rel_field_ref (id_field),
                                                        rel_make_field_str (patch_file_path)),
-                             0);
+                             rel_record_null);
           }
 
         rel_field_unref (id_field);
@@ -526,10 +541,168 @@ arch_print_added_file_diffs (int out_fd,
 
   for (x = 0; x < rel_n_records (report->added_files); ++x)
     {
-      arch_really_invoke_diff (1, "/dev/null", NULL, rel_peek_str (report->added_files, x, 2), NULL);
+      arch_really_invoke_diff (1, "/dev/null", NULL, rel_peek_str (report->added_files, x, 2), NULL, NULL);
     }
 }
 
+
+static void print_file_metadata (int out_fd, struct arch_changeset_report *
+                                 report, int escape_classes)
+{
+  if (rel_n_records(report->file_metadata_changed) > 0)
+    {
+      safe_printfmt (out_fd, "* file metadata changed\n\n");
+      arch_print_file_metadata_diffs (out_fd, report, escape_classes);
+      safe_printfmt (out_fd, "\n");
+    }
+}
+
+
+static void print_dir_metadata (int out_fd,
+                                struct arch_changeset_report * report,
+                                int escape_classes)
+{
+  if (rel_n_records(report->dir_metadata_changed) > 0)
+    {
+      safe_printfmt (out_fd, "* dir metadata changed\n\n");
+      arch_print_dir_metadata_diffs (out_fd, report, escape_classes);
+      safe_printfmt (out_fd, "\n");
+    }
+}
+
+static void print_diffs_new(int out_fd, struct arch_changeset_report * report)
+{
+  if (rel_n_records(report->patched_regular_files))
+    {
+      safe_printfmt (out_fd, "* modified files\n\n");
+      arch_print_file_diffs (out_fd, report, 0);
+      safe_printfmt (out_fd, "\n");
+    }
+  if (rel_n_records(report->added_files))
+    {
+      safe_printfmt (out_fd, "* added files\n\n");
+      arch_print_added_file_diffs (out_fd, report, 0);
+      safe_printfmt (out_fd, "\n");
+    }
+  print_removed_file_diffs(out_fd, report, NULL);
+}
+
+
+/**
+ * \brief Write custom diffs for to a file descriptor
+ *
+ * The changeset report is used to determine which files to diff, and the
+ * partial file paths.  The orig_dir and mod_dir are used to produce full
+ * paths.  diff is invoked to produce the output.
+ *
+ * \param report The report to use for output
+ * \param orig_dir The path to a copy of the ORIG tree
+ * \param mod_dir The path to a copy of the MOD tree
+ */
+static void
+print_custom_diffs (int out_fd, struct arch_changeset_report * report, t_uchar * orig_dir, t_uchar * mod_dir, t_uchar ** opts)
+{
+  if (rel_n_records(report->patched_regular_files))
+    {
+      int x;
+      rel_field key;
+      assoc_table orig_paths = rel_to_assoc (report->orig_files_index,  out_fd, 0);
+
+      safe_printfmt ( out_fd, "* modified files\n\n");
+      for (x = 0; x < rel_n_records (report->patched_regular_files); ++x)
+        {
+          t_uchar * id = str_save(0, rel_peek_str(report->patched_regular_files, x, 1)) /*report->patched_regular_files[x][1]*/;
+          t_uchar * id2 = str_save(0, rel_peek_str(report->patched_regular_files, x, 0)) /*report->patched_regular_files[x][1]*/;
+          key = rel_make_field_str(id);
+
+          t_uchar * orig_part_path = assoc_get_str_taking(orig_paths, key) /*assoc_ref(orig_paths, id)*/;
+          t_uchar * orig_path = file_name_in_vicinity(0, orig_dir, orig_part_path);
+          t_uchar * mod_path = file_name_in_vicinity(0, mod_dir, id2 /*report->patched_regular_files[x][0]*/);
+
+          arch_really_invoke_diff ( out_fd, orig_path,  orig_part_path, mod_path, id2 /*report->patched_regular_files[x][0]*/, (char **)opts);
+
+          rel_field_unref(key);
+          lim_free(0, mod_path);
+          lim_free(0, orig_path);
+          lim_free(0, id);
+          lim_free(0, id2);
+        }
+      safe_printfmt ( out_fd, "\n");
+      free_assoc_table (orig_paths);
+    }
+  if (rel_n_records(report->added_files))
+    {
+      int x;
+      safe_printfmt (1, "* added files\n\n");
+
+      for (x = 0; x < rel_n_records (report->added_files); ++x)
+        {
+          t_uchar *id = str_save(0, rel_peek_str(report->added_files, x, 2));
+          t_uchar *id2 = str_save(0, rel_peek_str(report->added_files, x, 0));
+          arch_really_invoke_diff ( out_fd, "/dev/null", NULL, id /*report->added_files[x][2]*/, id2 /*report->added_files[x][0]*/, (char**)opts);
+          lim_free(0, id);
+          lim_free(0, id2);
+        }
+      safe_printfmt ( out_fd, "\n");
+    }
+  print_removed_file_diffs ( out_fd, report, opts);
+}
+
+
+
+/**
+ * \brief Print the diffs for any removed files to output.
+ *
+ * If no files were removed, no output is produced
+ * \param report The changeset report to generate results for
+ * \param opts Additional parameters to pass to diff, or NULL
+ */
+static void
+print_removed_file_diffs (int out_fd, struct arch_changeset_report * report, t_uchar ** opts)
+{
+  if (rel_n_records(report->removed_files))
+    {
+      int x;
+      safe_printfmt ( out_fd, "* removed files\n\n");
+
+      for (x = 0; x < rel_n_records (report->removed_files); ++x)
+        {
+          t_uchar *id = str_save(0, rel_peek_str(report->removed_files, x, 2));
+          t_uchar *id2 = str_save(0, rel_peek_str(report->removed_files, x, 0));
+          arch_really_invoke_diff ( out_fd, id /*report->removed_files[x][2]*/, id2 /*report->removed_files[x][0]*/, "/dev/null", NULL, (char**)opts);
+          lim_free(0, id);
+          lim_free(0, id2);
+        }
+      safe_printfmt ( out_fd, "\n");
+    }
+}
+
+
+/**
+ * \brief Write a changeset report with custom diffs to a file descriptor
+ *
+ * \param report The report to use for output
+ * \param orig_dir The path to a copy of the ORIG tree
+ * \param mod_dir The path to a copy of the MOD tree
+ * \param escape_classes The types of characters that should be escaped
+ * \todo Consider reimplementing so that a custom changeset is generated, not
+ * custom diffs.
+ */
+void
+arch_print_changeset_custom_diffs (int  out_fd, struct arch_changeset_report * report, t_uchar * orig_dir, t_uchar * mod_dir, t_uchar ** opts, int escape_classes)
+{
+  print_file_metadata ( out_fd, report, escape_classes);
+  print_dir_metadata ( out_fd, report, escape_classes);
+  if (opts == NULL)
+    print_diffs_new ( out_fd, report);
+  else
+    print_custom_diffs ( out_fd, report, orig_dir, mod_dir, opts);
+}
+
+
+
+
+
 void
 arch_print_changeset_diffs (int out_fd, struct arch_changeset_report * report, int escape_classes)
 {
@@ -875,7 +1048,7 @@ report_unique_files_and_symlinks (rel_table * files, rel_table * symlinks,
                            rel_make_record_3_taking (rel_get_field (index, x, 0),
                                                      rel_get_field (index, x, 1),
                                                      rel_make_field_str (archive_path)),
-                           0);
+                           rel_record_null);
         }
       else
         {
@@ -883,7 +1056,7 @@ report_unique_files_and_symlinks (rel_table * files, rel_table * symlinks,
                            rel_make_record_3_taking (rel_get_field (index, x, 0),
                                                      rel_get_field (index, x, 1),
                                                      rel_make_field_str (archive_path)),
-                           0);
+                           rel_record_null);
         }
 
       lim_free (0, archive_path);
@@ -954,7 +1127,7 @@ find_renames (rel_table * out, rel_table common, assoc_table orig_dir_id_of, ass
           rel_add_records (out, rel_make_record_3_taking (orig_loc_field,
                                                           mod_loc_field,
                                                           id_field),
-                           0);
+                           rel_record_null);
         }
 
 
