@@ -69,6 +69,8 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
       "commit even if out of date") \
   OP (opt_file_list, 0, "file-list FILE", 1, \
       "commit only changes to files listed in FILE") \
+  OP (opt_write_revision, 0, "write-revision REVISION_FILE_SPEC", 1, \
+      "write the new tree id to REVISION_FILE_SPEC before committing") \
   OP (opt_unescaped, 0, "unescaped", 0, \
       "show filenames in unescaped form")
 
@@ -108,6 +110,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
   t_uchar * log_file = 0;
   t_uchar * log_text = 0;
   t_uchar * summary = 0;
+  t_uchar *revision_file_spec = 0;
   int strict = 0;
   int seal = 0;
   int fix = 0;
@@ -218,11 +221,18 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
             break;
           }
 
-	case opt_unescaped:
-	  {
-	    escape_classes = 0;
-	    break;
-	  }
+	    case opt_unescaped:
+	      {
+	        escape_classes = 0;
+	        break;
+	      }
+      
+        case opt_write_revision:
+          {
+             lim_free(0, revision_file_spec);
+             revision_file_spec = str_save(0, option->arg_string);
+             break;
+          }
         }
     }
 
@@ -296,7 +306,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
           goto usage_error;
 
         while (argx < argc)
-          rel_add_records (&file_list, rel_singleton_record_taking (rel_make_field_str (argv[argx++])), 0);
+          rel_add_records (&file_list, rel_singleton_record_taking (rel_make_field_str (argv[argx++])), rel_record_null);
       }
 
     if (!arch_valid_package_name (vsnspec, arch_maybe_archive, arch_req_version, 0))
@@ -361,7 +371,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
         }
     }
 
-
+    
     {
       struct arch_tree_lint_result * lint = 0;
       int status;
@@ -503,6 +513,36 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
     desired_level = arch_form_patch_level (desired_level_type, desired_n);
     revision = str_alloc_cat_many (0, version, "--", desired_level, str_end);
 
+    if (revision_file_spec) /* if revision file spec is specified, write the new revision to the file before committing */
+      {
+         t_uchar *file_name = 0;
+         t_uchar *content_spec = 0;
+         t_uchar *s = str_chr_index(revision_file_spec, ':');
+         
+         if (!s)
+           {
+             safe_printfmt (2, "%s: revision file spec does not contain file name separator ':'! commit aborted\n",  
+                            argv[0], revision_file_spec);
+             exit (1);
+           } 
+         file_name = str_save_n(0, revision_file_spec, s - revision_file_spec);
+         content_spec = str_save(0, s + 1); 
+         
+         int fd ;
+         fd = safe_open(file_name, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+         if (!fd)
+           {
+             safe_printfmt (2, "%s: cannot open file %s for writing revision! commit aborted\n",  
+                            argv[0], file_name);
+             exit (1);
+           }
+         safe_printfmt(fd, content_spec, revision);
+         safe_printfmt(fd, "\n");
+         safe_close(fd);
+         lim_free(0, file_name);
+         lim_free(0, content_spec);
+      }
+      
     arch_commit (1, arch, revision, tree_root, log, file_list, escape_classes);
 
     safe_printfmt (1, "* committed %s/%s\n", archive, revision);
@@ -521,6 +561,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
     lim_free (0, desired_level);
     lim_free (0, revision);
 
+    lim_free (0, revision_file_spec);
   }
 
   lim_free (0, default_archive);
