@@ -165,7 +165,7 @@ arch_cmd_library_revisions (t_uchar * program_name, int argc, char * argv[])
     t_uchar * version_spec = 0;
     t_uchar * archive = 0;
     t_uchar * version = 0;
-    rel_table revisions = 0;
+    rel_table revisions = rel_table_nil;
 
     if (argc == 2)
       version_spec = str_save (0, argv[1]);
@@ -206,31 +206,31 @@ arch_cmd_library_revisions (t_uchar * program_name, int argc, char * argv[])
             t_uchar * body = 0;
 
             if (full)
-              package_patch_level = arch_parse_package_name (arch_ret_non_archive, 0, revisions[x][0]);
+              package_patch_level = arch_parse_package_name (arch_ret_non_archive, 0, rel_peek_str (revisions, x, 0));
             else
-              package_patch_level = str_alloc_cat_many (0, version, "--", revisions[x][0], str_end);
+              package_patch_level = str_alloc_cat_many (0, version, "--", rel_peek_str (revisions, x, 0), str_end);
 
             log = arch_library_log (archive, package_patch_level);
 
-            arch_parse_log (0, &headers, &body, log);
+            arch_parse_log (0, &headers, (const t_uchar **)&body, log);
 
-            safe_printfmt (1, "%s\n", revisions[x][0]);
+            safe_printfmt (1, "%s\n", rel_peek_str (revisions, x, 0));
             if (date)
               {
-                t_uchar * d;
-                d = assoc_ref (headers, "standard-date");
+                const t_uchar * d;
+                d = assoc_get_str_taking (headers, rel_make_field_str ("standard-date"));
                 safe_printfmt (1, "    %s\n", (d ? d : (t_uchar *)"<no standard-date: header>"));
               }
             if (creator)
               {
-                t_uchar * c;
-                c = assoc_ref (headers, "creator");
+                const t_uchar * c;
+                c = assoc_get_str_taking (headers, rel_make_field_str ("creator"));
                 safe_printfmt (1, "    %s\n", (c ? c : (t_uchar *)"<no creator: header>"));
               }
             if (summary)
               {
-                t_uchar * s;
-                s = assoc_ref (headers, "summary");
+                const t_uchar * s;
+                s = assoc_get_str_taking (headers, rel_make_field_str ("summary"));
                 safe_printfmt (1, "    %s\n", (s ? s : (t_uchar *)"<no summary: header>"));
               }
 
