@@ -48,9 +48,9 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
   OP (opt_dot_listing, "l", "listing", 0, \
       "Keep .listing files up-to-date in this archive.") \
   OP (opt_tla, "t", "tla", 0, \
-      "Create a tla format archive (the default).") \
+      "Create a tla format archive.") \
   OP (opt_baz, "b", "baz", 0, \
-      "Create a baz format archive.") \
+      "Create a baz format archive (the default).") \
   OP (opt_signed, "s", "signed", 0, \
       "GPG sign the contents of this archive.")
 
