@@ -503,5 +503,51 @@ arch_check_arch (struct arch_archive * arch)
     }
 }
 
+
+/**
+ * \brief free current and return replacement
+ *
+ * Factors out blocks of temporary variables into one shot.
+ * \return replacement
+ * \param current the string to be freed.
+ * \param replacement the string to be returned
+ */
+t_uchar *
+str_replace (t_uchar *current, t_uchar *replacement)
+{
+    lim_free (0, current);
+    return replacement;
+} 
+
+int 
+arch_separate_arch_name_from_fqrvsn(const t_uchar *name, t_uchar **archive, t_uchar **fqrvsn)
+{
+
+    t_uchar *slash = 0;
+    t_uchar *tmp = str_save(0, name);
+    
+    lim_free(0, *archive);
+    lim_free(0, *fqrvsn);
+    if (0 == (slash = str_chr_index (tmp, '/')))
+      {
+        *archive = str_save(0, "");
+        *fqrvsn = str_save(0, name);
+        lim_free(0, tmp);
+        return 1;
+      }
+    else
+      { 
+        *slash = 0;
+        *archive = str_save(0, tmp);
+        *fqrvsn = str_save(0, slash + 1);
+        lim_free(0, tmp);
+        return 1;
+      }
+    return 0;
+}
+
+
+
+
 /* tag: Aaron Bentley Tue Dec 23 10:46:00 2003 (cmdutils.c)
  */
