@@ -44,24 +44,25 @@ static t_uchar * file_id (int * errn,
                           struct alloc_limits * limits,
                           enum ftag_method method,
                           int untagged_is_source,
-                          t_uchar * path,
+                          const t_uchar * path,
                           assoc_table id_tagging_shortcut,
                           struct stat * known_lstat,
                           assoc_table * explicit_skips);
-static int is_at_or_underneath_archdir (char * rel_file);
-static int filename_matches (regex_t * pattern, char * filename);
+static int is_at_or_underneath_archdir (const char * rel_file);
+static int filename_matches (regex_t * pattern,
+                             const char * filename);
 static t_uchar * explicit_id (int * errn,
                               assoc_table * skips,
                               struct alloc_limits * limits,
-                              t_uchar * arg_file,
-                              t_uchar * id_file,
-                              t_uchar * prefix,
-                              t_uchar * postfix);
+                              const t_uchar * arg_file,
+                              const t_uchar * id_file,
+                              const t_uchar * prefix,
+                              const t_uchar * postfix);
 static t_uchar * implicit_id (int * errn,
                               struct alloc_limits * limits,
-                              t_uchar * file,
-                              t_uchar * basename,
-                              t_uchar * prefix,
+                              const t_uchar * file,
+                              const t_uchar * basename,
+                              const t_uchar * prefix,
                               struct stat * statb,
                               assoc_table id_tagging_shortcut);
 static long smash_non_graphical (t_uchar * buf, long amt);
@@ -69,7 +70,8 @@ static long smash_non_graphical (t_uchar * buf, long amt);
 
 
 t_uchar *
-arch_log_file_id (t_uchar * archive, t_uchar * revision)
+arch_log_file_id (const t_uchar * archive,
+                  const t_uchar * revision)
 {
   t_uchar * log_file_path = 0;
   t_uchar * answer = 0;
@@ -83,7 +85,8 @@ arch_log_file_id (t_uchar * archive, t_uchar * revision)
 
 
 assoc_table
-arch_filenames_ids (rel_table * file_list, t_uchar * tree_root)
+arch_filenames_ids (rel_table * file_list,
+                    const t_uchar * tree_root)
 {
   struct arch_inventory_options options = {0, };
   int i = 0;
@@ -97,8 +100,8 @@ arch_filenames_ids (rel_table * file_list, t_uchar * tree_root)
 
   for (i = 0; i != file_size; ++ i)
     {
-      t_uchar * id = arch_inventory_id (options.method, 0, (* file_list) [i] [0], 0, 0, 0);
-      assoc_set (&id_list, id, (* file_list) [i] [0]);
+      t_uchar * id = arch_inventory_id (options.method, 0, rel_peek_str (*file_list, i, 0), 0, 0, 0);
+      assoc_set_taking (&id_list, rel_make_field_str (id), rel_get_field (*file_list, i, 0));
       lim_free (0, id);
     }
 
@@ -109,11 +112,11 @@ arch_filenames_ids (rel_table * file_list, t_uchar * tree_root)
 
 t_uchar *
 arch_inventory_id (enum arch_id_tagging_method method,
-                    int untagged_is_source,
-                    t_uchar * path,
-                    assoc_table id_tagging_shortcut,
-                    struct stat * known_lstat,
-                    assoc_table * explicit_skips)
+                   int untagged_is_source,
+                   const t_uchar * path,
+                   assoc_table id_tagging_shortcut,
+                   struct stat * known_lstat,
+                   assoc_table * explicit_skips)
 {
   int errn;
   t_uchar * answer;
@@ -187,6 +190,7 @@ arch_inventory_id (enum arch_id_tagging_method method,
   return answer;
 }
 
+
 t_uchar *
 arch_id_tagging_method_name (enum arch_id_tagging_method m)
 {
@@ -205,7 +209,7 @@ arch_id_tagging_method_name (enum arch_id_tagging_method m)
 
 
 enum arch_id_tagging_method
-arch_id_tagging_method_from_name (t_uchar * name)
+arch_id_tagging_method_from_name (const t_uchar * name)
 {
   if (!str_casecmp (name, "explicit"))
     return arch_explicit_id_tagging;
@@ -430,9 +434,8 @@ arch_default_id_tagging_method_contents (enum arch_id_tagging_method method)
 }
 
 
-
 t_uchar *
-arch_tree_id_tagging_method_file (t_uchar * tree_root)
+arch_tree_id_tagging_method_file (const t_uchar * tree_root)
 {
   t_uchar * ctl_dir;
   t_uchar * answer;
@@ -445,7 +448,9 @@ arch_tree_id_tagging_method_file (t_uchar * tree_root)
 
 
 enum arch_id_tagging_method
-arch_tree_id_tagging_method (enum arch_inventory_category * cat_var, t_uchar * tree_root, int strict)
+arch_tree_id_tagging_method (enum arch_inventory_category * cat_var,
+                             const t_uchar * tree_root,
+                             int strict)
 {
   struct arch_inventory_options options;
   enum arch_id_tagging_method answer;
@@ -463,8 +468,8 @@ arch_tree_id_tagging_method (enum arch_inventory_category * cat_var, t_uchar * t
 
 
 void
-arch_set_tree_id_tagging_method (t_uchar * tree_root,
-                              enum arch_id_tagging_method method)
+arch_set_tree_id_tagging_method (const t_uchar * tree_root,
+                                 enum arch_id_tagging_method method)
 {
   int errn;
   t_uchar * method_name;
@@ -580,7 +585,7 @@ arch_set_tree_id_tagging_method (t_uchar * tree_root,
 
 
 t_uchar *
-arch_explicit_id_file_for (t_uchar * path)
+arch_explicit_id_file_for (const t_uchar * path)
 {
   int errn;
   struct stat stat_buf;
@@ -665,7 +670,8 @@ arch_generate_id (void)
 
 
 void
-arch_add_explicit_id (t_uchar * path, t_uchar * id)
+arch_add_explicit_id (const t_uchar * path,
+                      const t_uchar * id)
 {
   t_uchar * id_file;
   t_uchar * id_dir;
@@ -692,7 +698,7 @@ arch_add_explicit_id (t_uchar * path, t_uchar * id)
 
 
 void
-arch_delete_explicit_id (t_uchar * path)
+arch_delete_explicit_id (const t_uchar * path)
 {
   t_uchar * id_file;
 
@@ -711,7 +717,8 @@ arch_delete_explicit_id (t_uchar * path)
 
 
 void
-arch_move_explicit_id (t_uchar * from, t_uchar * to)
+arch_move_explicit_id (const t_uchar * from,
+                       const t_uchar * to)
 {
   t_uchar * old_id_file;
   t_uchar * new_id_file;
@@ -731,7 +738,7 @@ arch_move_explicit_id (t_uchar * from, t_uchar * to)
 
 
 t_uchar *
-arch_strong_explicit_dflt_file (t_uchar * dir)
+arch_strong_explicit_dflt_file (const t_uchar * dir)
 {
   t_uchar * dot_arch_dir;
   t_uchar * answer;
@@ -743,8 +750,9 @@ arch_strong_explicit_dflt_file (t_uchar * dir)
   return answer;
 }
 
+
 t_uchar *
-arch_weak_explicit_dflt_file (t_uchar * dir)
+arch_weak_explicit_dflt_file (const t_uchar * dir)
 {
   t_uchar * dot_arch_dir;
   t_uchar * answer;
@@ -756,8 +764,9 @@ arch_weak_explicit_dflt_file (t_uchar * dir)
   return answer;
 }
 
+
 t_uchar *
-arch_dont_care_explicit_dflt_file (t_uchar * dir)
+arch_dont_care_explicit_dflt_file (const t_uchar * dir)
 {
   t_uchar * dot_arch_dir;
   t_uchar * answer;
@@ -769,8 +778,9 @@ arch_dont_care_explicit_dflt_file (t_uchar * dir)
   return answer;
 }
 
+
 int
-arch_is_dont_care_explicit_dflt_dir (t_uchar * dir)
+arch_is_dont_care_explicit_dflt_dir (const t_uchar * dir)
 {
   t_uchar * file = 0;
   int answer;
@@ -782,8 +792,9 @@ arch_is_dont_care_explicit_dflt_dir (t_uchar * dir)
   return answer;
 }
 
+
 void
-arch_delete_strong_explicit_default (t_uchar * dir)
+arch_delete_strong_explicit_default (const t_uchar * dir)
 {
   t_uchar * file;
 
@@ -801,7 +812,7 @@ arch_delete_strong_explicit_default (t_uchar * dir)
 
 
 void
-arch_delete_weak_explicit_default (t_uchar * dir)
+arch_delete_weak_explicit_default (const t_uchar * dir)
 {
   t_uchar * file;
 
@@ -817,8 +828,9 @@ arch_delete_weak_explicit_default (t_uchar * dir)
   lim_free (0, file);
 }
 
+
 void
-arch_delete_dont_care_explicit_default (t_uchar * dir)
+arch_delete_dont_care_explicit_default (const t_uchar * dir)
 {
   t_uchar * file;
 
@@ -836,7 +848,8 @@ arch_delete_dont_care_explicit_default (t_uchar * dir)
 
 
 void
-arch_set_strong_explicit_default (t_uchar * dir, t_uchar * id)
+arch_set_strong_explicit_default (const t_uchar * dir,
+                                  const t_uchar * id)
 {
   t_uchar * file;
   t_uchar * file_dir;
@@ -860,8 +873,10 @@ arch_set_strong_explicit_default (t_uchar * dir, t_uchar * id)
   lim_free (0, file_dir);
 }
 
+
 void
-arch_set_weak_explicit_default (t_uchar * dir, t_uchar * id)
+arch_set_weak_explicit_default (const t_uchar * dir,
+                                const t_uchar * id)
 {
   t_uchar * file;
   t_uchar * file_dir;
@@ -885,8 +900,9 @@ arch_set_weak_explicit_default (t_uchar * dir, t_uchar * id)
   lim_free (0, file_dir);
 }
 
+
 void
-arch_set_dont_care_explicit_default (t_uchar * dir)
+arch_set_dont_care_explicit_default (const t_uchar * dir)
 {
   t_uchar * file;
   t_uchar * file_dir;
@@ -913,7 +929,10 @@ arch_set_dont_care_explicit_default (t_uchar * dir)
  * static t_uchar * file_id (int * errn,
  *                           struct alloc_limits * limits,
  *                           enum ftag_method method,
- *                           t_uchar * path);
+ *                           const t_uchar * path,
+ *                           assoc_table id_tagging_shortcut,
+ *                           struct stat * known_lstat,
+ *                           assoc_table * explicit_skips)
  *
  * Return a newly allocated string containing the inventory id
  * of the file `path' using inventory method `method'.
@@ -927,7 +946,7 @@ file_id (int * errn,
          struct alloc_limits * limits,
          enum ftag_method method,
          int untagged_is_source,
-         t_uchar * path,
+         const t_uchar * path,
          assoc_table id_tagging_shortcut,
          struct stat * known_lstat,
          assoc_table * explicit_skips)
@@ -1128,7 +1147,7 @@ file_id (int * errn,
           if (!id_file)
             goto enomem_error;
 
-          skipped = (explicit_skips && !!assoc_ref (*explicit_skips, id_file));
+          skipped = (explicit_skips && !!assoc_get_str_taking (*explicit_skips, rel_make_field_str (id_file)));
 
           if (!skipped && (0 <= vu_lstat (errn, id_file, &stat_buf)))
             {
@@ -1142,7 +1161,7 @@ file_id (int * errn,
             }
 
           if (!skipped && explicit_skips)
-            assoc_set (explicit_skips, id_file, "yes");
+            assoc_set_taking (explicit_skips, rel_make_field_str (id_file), rel_make_field_str ("yes"));
 
           if (*errn == ENOENT)
             {
@@ -1169,13 +1188,15 @@ file_id (int * errn,
 
 
 t_uchar *
-arch_id_from_explicit_file (int *errn, t_uchar * path)
+arch_id_from_explicit_file (int *errn,
+                            const t_uchar * path)
 {
   return explicit_id (errn, 0, 0, 0, path, "x_", 0);
 }
 
+
 static int
-is_at_or_underneath_archdir (char * rel_file)
+is_at_or_underneath_archdir (const char * rel_file)
 {
   static int compiled = 0;
   static regex_t pattern = {0,};
@@ -1192,8 +1213,10 @@ is_at_or_underneath_archdir (char * rel_file)
   return filename_matches (&pattern, rel_file);
 }
 
+
 static int
-filename_matches (regex_t * pattern, char * filename)
+filename_matches (regex_t * pattern,
+                  const char * filename)
 {
   int answer;
 
@@ -1212,12 +1235,12 @@ filename_matches (regex_t * pattern, char * filename)
 
 static t_uchar *
 explicit_id (int * errn,
-              assoc_table * skips,
-              struct alloc_limits * limits,
-              t_uchar * arg_file,
-              t_uchar * id_file,
-              t_uchar * prefix,
-              t_uchar * postfix)
+             assoc_table * skips,
+             struct alloc_limits * limits,
+             const t_uchar * arg_file,
+             const t_uchar * id_file,
+             const t_uchar * prefix,
+             const t_uchar * postfix)
 {
   int id_fd;
   t_uchar * answer;
@@ -1226,7 +1249,7 @@ explicit_id (int * errn,
   int ign;
 
 
-  if (skips && assoc_ref (*skips, id_file))
+  if (skips && assoc_get_str_taking (*skips, rel_make_field_str (id_file)))
     {
       *errn = ENOENT;
       return 0;
@@ -1237,7 +1260,7 @@ explicit_id (int * errn,
   if (id_fd < 0)
     {
       if (skips)
-        assoc_set (skips, id_file, "yes");
+        assoc_set_taking (skips, rel_make_field_str (id_file), rel_make_field_str ("yes"));
       return 0;
     }
 
@@ -1300,14 +1323,15 @@ explicit_id (int * errn,
 
 static t_uchar *
 implicit_id (int * errn,
-              struct alloc_limits * limits,
-              t_uchar * file,
-              t_uchar * basename,
-              t_uchar * prefix,
-              struct stat * statb,
-              assoc_table id_tagging_shortcut)
+             struct alloc_limits * limits,
+             const t_uchar * file,
+             const t_uchar * basename,
+             const t_uchar * prefix,
+             struct stat * statb,
+             assoc_table id_tagging_shortcut)
 {
   int file_fd;
+  struct stat file_stat_buf;
   char buf[1025];
   int amt;
   int line;
@@ -1316,7 +1340,7 @@ implicit_id (int * errn,
   if (id_tagging_shortcut)
     {
       t_uchar * signature = arch_statb_inode_sig (statb);
-      t_uchar * cached_id = assoc_ref (id_tagging_shortcut, signature);
+      const t_uchar * cached_id = assoc_get_str_taking (id_tagging_shortcut, rel_make_field_str (signature));
 
       lim_free (0, signature);
       if (cached_id && (cached_id[0] == 'i') && (cached_id[1] == '_'))
@@ -1355,6 +1379,9 @@ implicit_id (int * errn,
   file_fd = vu_open (errn, file, O_RDONLY, 0);
   if (file_fd < 0)
     return 0;
+  
+  if (0 > vu_fstat (errn, file_fd, &file_stat_buf))
+      goto error_return;
 
   for (bottom = 1; bottom >= 0; --bottom)
     {
@@ -1373,15 +1400,15 @@ implicit_id (int * errn,
         }
       else
         {
-          struct stat file_stat_buf;
           char * x;
 
-          if (0 > vu_fstat (errn, file_fd, &file_stat_buf))
-            goto error_return;
           if (file_stat_buf.st_size > sizeof (buf))
             amt = sizeof (buf);
           else
             continue;
+          /* Yes, this is a off by one error. However changing it
+           * breaks existing file-ids
+           */
           if (0 > vu_lseek (errn, file_fd, -1026, SEEK_END))
             goto error_return;
           amt = vu_read_retry (errn, file_fd, buf, sizeof (buf));
@@ -1463,12 +1490,10 @@ implicit_id (int * errn,
               if (!eol)
                 /* end of buffer no eol */
                 {
-                  if (bottom)
-                    eol = buf + amt;
-                  else
-                    {
-                      panic("Top-of-file arch tag crosses 1k boundary");
-                    }
+                  eol = buf + amt;
+                  if (!bottom)
+                    if (file_stat_buf.st_size > sizeof (buf))
+                      safe_printfmt (2, "Warning: top-of-file truncated tag in: %s\n", file);
                 }
 
               if (0 == (eol - (t_uchar *)(buf + line)))
@@ -1524,6 +1549,18 @@ smash_non_graphical (t_uchar * buf, long amt)
   return amt;
 }
 
+
+int
+str_cmp_suffix_n(t_uchar * a_string, t_uchar * b_string, int length)
+{
+    int len_a = str_length(a_string);
+    int len_b = str_length(b_string);
+    int cmp_length = (len_a < len_b) ? len_a : len_b;
+    cmp_length = (cmp_length < length) ? cmp_length : length;
+    return str_cmp(a_string + len_a - cmp_length, b_string + len_b - cmp_length);
+}
+
+
 
 /* tag: Tom Lord Wed May 14 07:20:26 2003 (inv-tags.c)
  */
