@@ -170,7 +170,7 @@ source_inventory_callback (const t_uchar * path,
 {
   rel_table * answer = (rel_table *)closure;
 
-  rel_add_records (answer, rel_make_record_2_taking (rel_make_field_str (path), rel_make_field_str (id)), 0);
+  rel_add_records (answer, rel_make_record_2_taking (rel_make_field_str (path), rel_make_field_str (id)), rel_record_null);
 }
 
 
@@ -186,7 +186,7 @@ source_inventory_files_callback (const t_uchar * path,
   rel_table * answer = (rel_table *)closure;
 
   if (!S_ISDIR (statb->st_mode))
-    rel_add_records (answer, rel_make_record_2_taking (rel_make_field_str (path), rel_make_field_str (id)), 0);
+    rel_add_records (answer, rel_make_record_2_taking (rel_make_field_str (path), rel_make_field_str (id)), rel_record_null);
 }
 
 
@@ -986,6 +986,10 @@ is_control_file (const char * rel_file, const char * filename)
   return filename_matches (&control_pattern, rel_file);
 }
 
+int arch_is_control_file (const char * rel_file, const char * filename)
+{
+    return is_control_file (rel_file, filename);
+}
 
 static int
 is_nested_tree (const char * path)
