@@ -137,7 +137,7 @@ arch_library_add_choice (const t_uchar * archive,
           exit (2);
         }
 
-      rel_add_records (&search_path, rel_singleton_record_taking (rel_get_field (full_user_path, x, 0)), 0);
+      rel_add_records (&search_path, rel_singleton_record_taking (rel_get_field (full_user_path, x, 0)), rel_record_null);
     }
 
   else
@@ -245,7 +245,7 @@ make_search_path (rel_table full_user_path,
 
       if (require_greedy && !arch_library_is_greedy(rel_peek_str (full_user_path, x, 0)))
 	continue;
-      rel_add_records (&search_path, rel_singleton_record_taking (rel_get_field (full_user_path, x, 0)), 0);
+      rel_add_records (&search_path, rel_singleton_record_taking (rel_get_field (full_user_path, x, 0)), rel_record_null);
     }
 
   return search_path;
@@ -450,7 +450,7 @@ arch_library_add (int chatter_fd,
 
       /* this is the first patch missing
        */
-      rel_add_records (&revisions,  rel_singleton_record_taking (rel_make_field_str (revision)), 0);
+      rel_add_records (&revisions,  rel_singleton_record_taking (rel_make_field_str (revision)), rel_record_null);
 
       arch_chatter (chatter_fd, "* searching ancestor revision in library in archive %s\n", ancestor_archive);
 
@@ -464,7 +464,7 @@ arch_library_add (int chatter_fd,
           /* add revision to list
            */
 
-          rel_add_records (&revisions,  rel_singleton_record_taking (rel_make_field_str (anc_rev)), 0);
+          rel_add_records (&revisions,  rel_singleton_record_taking (rel_make_field_str (anc_rev)), rel_record_null);
 
           /* look in the library for the immediate ancestor
            */
