@@ -70,7 +70,7 @@ arch_find_local_copy (int chatter_fd,
       error = arch_run_hook (hook,
                              "ARCH_ARCHIVE", archive,
                              "ARCH_REVISION", revision,
-                             0);
+                             (t_uchar*)0);
       
       if (error)
         {
