@@ -124,7 +124,7 @@ filter_present_logs (t_uchar * tree_root, struct arch_archive * arch, t_uchar *
         log = arch_archive_log (arch, revision);
 
         if (is_a_patch_present(has, log))
-            rel_add_records (&to_skip, rel_copy_record (rel_peek_record (unfiltered, x)), 0);
+            rel_add_records (&to_skip, rel_copy_record (rel_peek_record (unfiltered, x)), rel_record_null);
         lim_free (0, revision);
       }
     rel_free_table (has);
