@@ -380,7 +380,7 @@ arch_log_versions (const t_uchar * tree_root,
         continue;
 
       if (arch_valid_package_name (rel_peek_str (dir_listing, x, 0), arch_no_archive, arch_req_category, 0))
-        rel_add_records (&categories, rel_copy_record (rel_peek_record (dir_listing, x)), 0);
+        rel_add_records (&categories, rel_copy_record (rel_peek_record (dir_listing, x)), rel_record_null);
     }
   rel_free_table (dir_listing);
 
@@ -422,7 +422,7 @@ arch_log_versions (const t_uchar * tree_root,
 
               branch_path = file_name_in_vicinity (0, cat_dir, maybe_branch);
               rel_field_ref (maybe_branch_field);
-              rel_add_records (&branches, rel_make_record_2_taking (maybe_branch_field, rel_make_field_str (branch_path)), 0);
+              rel_add_records (&branches, rel_make_record_2_taking (maybe_branch_field, rel_make_field_str (branch_path)), rel_record_null);
               lim_free (0, branch_path);
             }
 
@@ -475,7 +475,7 @@ arch_log_versions (const t_uchar * tree_root,
 
               version_path = file_name_in_vicinity (0, branch_dir, maybe_version);
               rel_field_ref (maybe_version_field);
-              rel_add_records (&versions, rel_make_record_2_taking (maybe_version_field, rel_make_field_str (version_path)), 0);
+              rel_add_records (&versions, rel_make_record_2_taking (maybe_version_field, rel_make_field_str (version_path)), rel_record_null);
               lim_free (0, version_path);
             }
 
@@ -528,7 +528,7 @@ arch_log_versions (const t_uchar * tree_root,
               t_uchar * fqversion;
 
               fqversion = arch_fully_qualify (maybe_archive, version);
-              rel_add_records (&log_versions, rel_make_record_2_taking (rel_make_field_str (fqversion), rel_make_field_str (patch_log_path)), 0);
+              rel_add_records (&log_versions, rel_make_record_2_taking (rel_make_field_str (fqversion), rel_make_field_str (patch_log_path)), rel_record_null);
               lim_free (0, fqversion);
             }
 
@@ -597,9 +597,9 @@ arch_logs (const t_uchar * tree_root,
            const t_uchar * version,
            int full)
 {
-  rel_table answer;             /* [0] == lvl, [2] == path */
-  t_uchar * log_dir;
-  rel_table dir_listing;
+  rel_table answer = rel_table_nil;             /* [0] == lvl, [2] == path */
+  t_uchar * log_dir = 0;
+  rel_table dir_listing = rel_table_nil;
   int lim;
   int x;
 
@@ -623,14 +623,14 @@ arch_logs (const t_uchar * tree_root,
 
           if (!full)
             {
-              rel_add_records (&answer, rel_make_record_2_taking (rel_get_field (dir_listing, x, 0), rel_make_field_str (path)), 0);
+              rel_add_records (&answer, rel_make_record_2_taking (rel_get_field (dir_listing, x, 0), rel_make_field_str (path)), rel_record_null);
             }
           else
             {
               t_uchar * full_name = 0;
 
               full_name = str_alloc_cat_many (0, archive, "/", version, "--", rel_peek_str (dir_listing, x, 0), str_end);
-              rel_add_records (&answer, rel_make_record_2_taking (rel_make_field_str (full_name), rel_make_field_str (path)), 0);
+              rel_add_records (&answer, rel_make_record_2_taking (rel_make_field_str (full_name), rel_make_field_str (path)), rel_record_null);
               lim_free (0, full_name);
             }
 
@@ -640,6 +640,7 @@ arch_logs (const t_uchar * tree_root,
 
  leave:
   lim_free (0, log_dir);
+  rel_free_table (dir_listing);
 
   if (full)
     arch_sort_table_by_name_field (0, answer, 0);
@@ -774,7 +775,7 @@ arch_parse_log (rel_table * headers_list,
       if (headers)
         assoc_set_taking (headers, rel_make_field_str (field_name), rel_make_field_str (field_value));
       if (headers_list)
-        rel_add_records (headers_list, rel_make_record_2_taking (rel_make_field_str (field_name), rel_make_field_str (field_value)), 0);
+        rel_add_records (headers_list, rel_make_record_2_taking (rel_make_field_str (field_name), rel_make_field_str (field_value)), rel_record_null);
 
       lim_free (0, field_name);
       lim_free (0, field_value);
