@@ -21,14 +21,14 @@
 #include "tla/libfsutils/file-contents.h"
 #include "tla/libarch/archives.h"
 #include "tla/libarch/pfs-dav.h"
-#include "tla/libneon/ne_session.h"
-#include "tla/libneon/ne_basic.h"
-#include "tla/libneon/ne_request.h"
-#include "tla/libneon/ne_auth.h"
-#include "tla/libneon/ne_props.h"
-#include "tla/libneon/ne_uri.h"
-#include "tla/libneon/ne_redirect.h"
-#include "tla/libneon/ne_socket.h"
+#include "libneon/src/ne_session.h"
+#include "libneon/src/ne_basic.h"
+#include "libneon/src/ne_request.h"
+#include "libneon/src/ne_auth.h"
+#include "libneon/src/ne_props.h"
+#include "libneon/src/ne_uri.h"
+#include "libneon/src/ne_redirect.h"
+#include "libneon/src/ne_socket.h"
 
 
 
@@ -78,24 +78,58 @@ struct arch_pfs_dav_session
 
 
 /* __STDC__ prototypes for static functions */
-static t_uchar * pfs_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static rel_table pfs_directory_files (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static int pfs_file_exists (struct arch_pfs_session * p, t_uchar * path);
-static int pfs_get_file (struct arch_pfs_session * p, int out_fd, t_uchar * path, int soft_errors);
-static int pfs_put_file (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int in_fd, int soft_errors);
-static int pfs_mkdir (struct arch_pfs_session * p, t_uchar * path, mode_t mode, int soft_errors);
-static int pfs_rename (struct arch_pfs_session * p, t_uchar ** errstr, t_uchar * from, t_uchar * to, int soft_errors);
-static int pfs_is_dir (struct arch_pfs_session * p, t_uchar * path);
-static int dav_is_dir (struct arch_pfs_dav_session * pfs, char * dir);
-static int pfs_rmdir (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static int pfs_rm (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static void dav_is_dir_results (void * userdata, const char * uri, const ne_prop_result_set * set);
-static int dav_client_cwd (struct arch_pfs_dav_session * pfs, t_uchar * path);
-static int dav_client_auth (void * userdata, const char * realm, int attempt,
-                            char * username, char * password);
-static t_uchar * abs_path (t_uchar * cwd, t_uchar * path);
+static t_uchar * pfs_file_contents (struct arch_pfs_session * p,
+                                    const t_uchar * path,
+                                    int soft_errors);
+static rel_table pfs_directory_files (struct arch_pfs_session * p,
+                                      const t_uchar * path,
+                                      int soft_errors);
+static int pfs_file_exists (struct arch_pfs_session * p,
+                            const t_uchar * path);
+static int pfs_get_file (struct arch_pfs_session * p,
+                         int out_fd,
+                         const t_uchar * path,
+                         int soft_errors);
+static int pfs_put_file (struct arch_pfs_session * p,
+                         const t_uchar * path,
+                         mode_t perms,
+                         int in_fd,
+                         int soft_errors);
+static int pfs_mkdir (struct arch_pfs_session * p,
+                      const t_uchar * path,
+                      mode_t mode,
+                      int soft_errors);
+static int pfs_rename (struct arch_pfs_session * p,
+                       t_uchar ** errstr,
+                       const t_uchar * from,
+                       const t_uchar * to,
+                       int soft_errors);
+static int pfs_is_dir (struct arch_pfs_session * p,
+                       const t_uchar * path);
+static int dav_is_dir (struct arch_pfs_dav_session * pfs,
+                       const char * dir);
+static int pfs_rmdir (struct arch_pfs_session * p,
+                      const t_uchar * path,
+                      int soft_errors);
+static int pfs_rm (struct arch_pfs_session * p,
+                   const t_uchar * path,
+                   int soft_errors);
+static void dav_is_dir_results (void * userdata,
+                                const char * uri,
+                                const ne_prop_result_set * set);
+static int dav_client_cwd (struct arch_pfs_dav_session * pfs,
+                           const t_uchar * path);
+static int dav_client_auth (void * userdata,
+                            const char * realm,
+                            int attempt,
+                            char * username,
+                            char * password);
+static t_uchar * abs_path (const t_uchar * cwd,
+                           const t_uchar * path);
 static t_uchar * dirfold (t_uchar *dir);
-static void results (void * userdata, const char * uri, const ne_prop_result_set * set);
+static void results (void * userdata,
+                     const char * uri,
+                     const ne_prop_result_set * set);
 
 
 
@@ -121,7 +155,7 @@ struct arch_pfs_vtable dav_pfs_fns =
 
 
 int
-arch_pfs_dav_supported_protocol (t_uchar * uri)
+arch_pfs_dav_supported_protocol (const t_uchar * uri)
 {
   if (!str_cmp_prefix ("http:", uri) ||
       !str_cmp_prefix ("https:", uri))
@@ -132,7 +166,7 @@ arch_pfs_dav_supported_protocol (t_uchar * uri)
 
 
 struct arch_pfs_session *
-arch_pfs_dav_connect (t_uchar * uri)
+arch_pfs_dav_connect (const t_uchar * uri)
 {
   int ign;
   struct arch_pfs_dav_session * answer = 0;
@@ -287,8 +321,11 @@ arch_pfs_dav_connect (t_uchar * uri)
   return (struct arch_pfs_session *)answer;
 }
 
+
 static t_uchar *
-pfs_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_file_contents (struct arch_pfs_session * p,
+                   const t_uchar * path,
+                   int soft_errors)
 {
   struct arch_pfs_dav_session * pfs = (struct arch_pfs_dav_session *)p;
   t_uchar * tmp_path = 0;
@@ -323,14 +360,17 @@ pfs_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
   return answer;
 }
 
+
 static rel_table
-pfs_directory_files (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_directory_files (struct arch_pfs_session * p,
+                     const t_uchar * path,
+                     int soft_errors)
 {
   struct arch_pfs_dav_session * pfs = (struct arch_pfs_dav_session *)p;
   int ne_err;
   t_uchar * dir = 0;
   t_uchar * file_string = 0;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
 
   if (pfs->sess_opts.dav_class1)
     {
@@ -375,12 +415,14 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path, int soft_error
   return answer;
 }
 
+
 static int
-pfs_file_exists (struct arch_pfs_session * p, t_uchar * path)
+pfs_file_exists (struct arch_pfs_session * p,
+                 const t_uchar * path)
 {
   t_uchar * path_dir = 0;
   t_uchar * path_tail = 0;
-  rel_table files = 0;
+  rel_table files = rel_table_nil;
   int answer = 0;
   int x;
 
@@ -398,7 +440,7 @@ pfs_file_exists (struct arch_pfs_session * p, t_uchar * path)
 
   for (x = 0; x < rel_n_records (files); ++x)
     {
-      if (!str_cmp (path_tail, files[x][0]))
+      if (!str_cmp (path_tail, rel_peek_str (files, x, 0)))
         {
           answer = 1;
           break;
@@ -413,8 +455,12 @@ pfs_file_exists (struct arch_pfs_session * p, t_uchar * path)
   return answer;
 }
 
+
 static int
-pfs_get_file (struct arch_pfs_session * p, int out_fd, t_uchar * path, int soft_errors)
+pfs_get_file (struct arch_pfs_session * p,
+              int out_fd,
+              const t_uchar * path,
+              int soft_errors)
 {
   struct arch_pfs_dav_session * pfs = (struct arch_pfs_dav_session *)p;
   t_uchar * dav_path = 0;
@@ -441,7 +487,11 @@ pfs_get_file (struct arch_pfs_session * p, int out_fd, t_uchar * path, int soft_
 
 
 static int
-pfs_put_file (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int in_fd, int soft_errors)
+pfs_put_file (struct arch_pfs_session * p,
+              const t_uchar * path,
+              mode_t perms,
+              int in_fd,
+              int soft_errors)
 {
   struct arch_pfs_dav_session * pfs = (struct arch_pfs_dav_session *)p;
   int ne_err;
@@ -472,7 +522,10 @@ pfs_put_file (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int in_
 
 
 static int
-pfs_mkdir (struct arch_pfs_session * p, t_uchar * path, mode_t mode, int soft_errors)
+pfs_mkdir (struct arch_pfs_session * p,
+           const t_uchar * path,
+           mode_t mode,
+           int soft_errors)
 {
   struct arch_pfs_dav_session * pfs = (struct arch_pfs_dav_session *)p;
   int ne_err;
@@ -500,7 +553,11 @@ pfs_mkdir (struct arch_pfs_session * p, t_uchar * path, mode_t mode, int soft_er
 
 
 static int
-pfs_rename (struct arch_pfs_session * p, t_uchar ** errstr, t_uchar * from, t_uchar * to, int soft_errors)
+pfs_rename (struct arch_pfs_session * p,
+            t_uchar ** errstr,
+            const t_uchar * from,
+            const t_uchar * to,
+            int soft_errors)
 {
   struct arch_pfs_dav_session * pfs = (struct arch_pfs_dav_session *)p;
   int ne_err;
@@ -565,8 +622,10 @@ pfs_rename (struct arch_pfs_session * p, t_uchar ** errstr, t_uchar * from, t_uc
   return result;
 }
 
+
 static int
-pfs_is_dir (struct arch_pfs_session * p, t_uchar * path)
+pfs_is_dir (struct arch_pfs_session * p,
+            const t_uchar * path)
 {
   struct arch_pfs_dav_session * pfs = (struct arch_pfs_dav_session *)p;
   t_uchar * abs = 0;
@@ -581,8 +640,10 @@ pfs_is_dir (struct arch_pfs_session * p, t_uchar * path)
   return answer;
 }
 
+
 static int
-dav_is_dir (struct arch_pfs_dav_session * pfs, char * dir)
+dav_is_dir (struct arch_pfs_dav_session * pfs,
+            const char * dir)
 {
   char * filename;
   char * parent;
@@ -594,7 +655,9 @@ dav_is_dir (struct arch_pfs_dav_session * pfs, char * dir)
   filename = str_chr_rindex (parent, '/');
   filename[1] = 0;
 
-  data.uri = dir;
+  /* `const' discarded with hopes that libneon isn't obnoxious 
+   */
+  data.uri = (char *)dir;
   data.is_dir = 0;
 
   if((ne_err = ne_simple_propfind (pfs->sess, parent, NE_DEPTH_ONE, ls_props, dav_is_dir_results, &data)))
@@ -610,8 +673,11 @@ dav_is_dir (struct arch_pfs_dav_session * pfs, char * dir)
   return answer;
 }
 
+
 static int
-pfs_rmdir (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_rmdir (struct arch_pfs_session * p,
+           const t_uchar * path,
+           int soft_errors)
 {
   struct arch_pfs_dav_session * pfs = (struct arch_pfs_dav_session *)p;
   int ne_err;
@@ -638,8 +704,11 @@ pfs_rmdir (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
   return answer;
 }
 
+
 static int
-pfs_rm (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_rm (struct arch_pfs_session * p,
+        const t_uchar * path,
+        int soft_errors)
 {
   struct arch_pfs_dav_session * pfs = (struct arch_pfs_dav_session *)p;
   int ne_err;
@@ -664,12 +733,12 @@ pfs_rm (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
 }
 
 
-
 
 
-
 static void
-dav_is_dir_results (void * userdata, const char * uri, const ne_prop_result_set * set)
+dav_is_dir_results (void * userdata,
+                    const char * uri,
+                    const ne_prop_result_set * set)
 {
   struct dav_is_dir_ls_data * data = (struct dav_is_dir_ls_data *)userdata;
   int len;
@@ -684,8 +753,10 @@ dav_is_dir_results (void * userdata, const char * uri, const ne_prop_result_set
     }
 }
 
+
 static int
-dav_client_cwd (struct arch_pfs_dav_session * pfs, t_uchar * path)
+dav_client_cwd (struct arch_pfs_dav_session * pfs,
+                const t_uchar * path)
 {
   int ne_err;
   t_uchar * tmp;
@@ -737,8 +808,11 @@ dav_client_cwd (struct arch_pfs_dav_session * pfs, t_uchar * path)
 
 
 static int
-dav_client_auth (void * userdata, const char * realm, int attempt,
-                 char * username, char * password)
+dav_client_auth (void * userdata,
+                 const char * realm,
+                 int attempt,
+                 char * username,
+                 char * password)
 {
   struct authinfo *auth;
   int len;
@@ -757,7 +831,8 @@ dav_client_auth (void * userdata, const char * realm, int attempt,
 
 
 static t_uchar *
-abs_path (t_uchar * cwd, t_uchar * path)
+abs_path (const t_uchar * cwd,
+          const t_uchar * path)
 {
   t_uchar * file;
   t_uchar * tmp;
@@ -815,8 +890,11 @@ dirfold (t_uchar *dir)
   return dir;
 }
 
+
 static void
-results (void * userdata, const char * uri, const ne_prop_result_set * set)
+results (void * userdata,
+         const char * uri,
+         const ne_prop_result_set * set)
 {
   int n;
   char * file, * tmp;
