@@ -28,7 +28,6 @@
 #include "tla/libfsutils/copy-file.h"
 #include "tla/libfsutils/tmp-files.h"
 #include "tla/libfsutils/file-contents.h"
-#include "tla/libawk/trim.h"
 #include "tla/libarch/archives.h"
 #include "tla/libarch/pfs-ftp.h"
 
@@ -51,16 +50,40 @@ struct arch_pfs_ftp_session
 
 
 /* __STDC__ prototypes for static functions */
-static int pfs_ftp_file_exists (struct arch_pfs_session * p, t_uchar * path);
-static int pfs_ftp_is_dir (struct arch_pfs_session * p, t_uchar * path);
-static t_uchar * pfs_ftp_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static int pfs_ftp_get_file (struct arch_pfs_session * p, int file_out_fd, t_uchar * path, int soft_errors);
-static rel_table pfs_ftp_directory_files (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static int pfs_ftp_put_file (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int file_in_fd, int soft_errors);
-static int pfs_ftp_mkdir (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int soft_errors);
-static int pfs_ftp_rename (struct arch_pfs_session * p, t_uchar ** errstr, t_uchar * from, t_uchar * to, int soft_errors);
-static int pfs_ftp_rmdir (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
-static int pfs_ftp_rm (struct arch_pfs_session * p, t_uchar * path, int soft_errors);
+static int pfs_ftp_file_exists (struct arch_pfs_session * p,
+                                const t_uchar * path);
+static int pfs_ftp_is_dir (struct arch_pfs_session * p,
+                           const t_uchar * path);
+static t_uchar * pfs_ftp_file_contents (struct arch_pfs_session * p,
+                                        const t_uchar * path,
+                                        int soft_errors);
+static int pfs_ftp_get_file (struct arch_pfs_session * p,
+                             int file_out_fd,
+                             const t_uchar * path,
+                             int soft_errors);
+static rel_table pfs_ftp_directory_files (struct arch_pfs_session * p,
+                                          const t_uchar * path,
+                                          int soft_errors);
+static int pfs_ftp_put_file (struct arch_pfs_session * p,
+                             const t_uchar * path,
+                             mode_t perms,
+                             int file_in_fd,
+                             int soft_errors);
+static int pfs_ftp_mkdir (struct arch_pfs_session * p,
+                          const t_uchar * path,
+                          mode_t perms,
+                          int soft_errors);
+static int pfs_ftp_rename (struct arch_pfs_session * p,
+                           t_uchar ** errstr,
+                           const t_uchar * from,
+                           const t_uchar * to,
+                           int soft_errors);
+static int pfs_ftp_rmdir (struct arch_pfs_session * p,
+                          const t_uchar * path,
+                          int soft_errors);
+static int pfs_ftp_rm (struct arch_pfs_session * p,
+                       const t_uchar * path,
+                       int soft_errors);
 static int ftp_client_greet (alloc_limits limits,
                              int * errn,
                              int * code,
@@ -75,8 +98,8 @@ static int ftp_client_login (alloc_limits limits,
                              long * text_len,
                              int in_fd,
                              int out_fd,
-                             t_uchar * name,
-                             t_uchar * passwd);
+                             const t_uchar * name,
+                             const t_uchar * passwd);
 static int ftp_client_read_reply (alloc_limits limits,
                                   int * errn,
                                   int * code,
@@ -89,7 +112,13 @@ static int ftp_client_read_simple_cmd_reply (alloc_limits limit,
                                              t_uchar ** text,
                                              long * text_len,
                                              int in_fd);
-static int ftp_client_printfmt (int * errn, int fd, char * fmt, ...);
+static int ftp_client_printfmt (int * errn, int fd, const char * fmt, ...);
+static int ftp_client_pasv_parse_equal (const char *text,
+                                        t_ulong *h,
+                                        t_uint16 *p);
+static int ftp_client_pasv_parse_parenthesis (const char *text,
+                                              t_ulong *h,
+                                              t_uint16 *p);
 static int ftp_client_pasv (alloc_limits limits,
                             int * errn,
                             int * code,
@@ -113,7 +142,7 @@ static int ftp_client_begin_retr (alloc_limits limits,
                                   long * text_len,
                                   int in_fd,
                                   int out_fd,
-                                  t_uchar * path);
+                                  const t_uchar * path);
 static int ftp_client_end_retr (alloc_limits limits,
                                 int * errn,
                                 int * code,
@@ -128,7 +157,7 @@ static int ftp_client_begin_stor (alloc_limits limits,
                                   long * text_len,
                                   int in_fd,
                                   int out_fd,
-                                  t_uchar * path);
+                                  const t_uchar * path);
 static int ftp_client_end_stor (alloc_limits limits,
                                 int * errn,
                                 int * code,
@@ -144,7 +173,7 @@ static int ftp_client_begin_nlst (alloc_limits limits,
                                   int in_fd,
                                   int out_fd,
                                   int is_wu_ftp,
-                                  t_uchar * path,
+                                  const t_uchar * path,
                                   t_ulong host_addr,
                                   t_uint16 port);
 static int ftp_client_end_nlst (alloc_limits limits,
@@ -161,7 +190,7 @@ static int ftp_client_mkd (alloc_limits limits,
                            long * text_len,
                            int in_fd,
                            int out_fd,
-                           t_uchar * path);
+                           const t_uchar * path);
 static int ftp_client_rename (alloc_limits limits,
                               int * errn,
                               int * code,
@@ -169,8 +198,8 @@ static int ftp_client_rename (alloc_limits limits,
                               long * text_len,
                               int in_fd,
                               int out_fd,
-                              t_uchar * from,
-                              t_uchar * to);
+                              const t_uchar * from,
+                              const t_uchar * to);
 static int ftp_client_dele (alloc_limits limits,
                             int * errn,
                             int * code,
@@ -178,7 +207,7 @@ static int ftp_client_dele (alloc_limits limits,
                             long * text_len,
                             int in_fd,
                             int out_fd,
-                            t_uchar * path);
+                            const t_uchar * path);
 static int ftp_client_rmd (alloc_limits limits,
                            int * errn,
                            int * code,
@@ -186,7 +215,7 @@ static int ftp_client_rmd (alloc_limits limits,
                            long * text_len,
                            int in_fd,
                            int out_fd,
-                           t_uchar * path);
+                           const t_uchar * path);
 static int ftp_client_cwd (alloc_limits limits,
                            int * errn,
                            int * code,
@@ -194,7 +223,7 @@ static int ftp_client_cwd (alloc_limits limits,
                            long * text_len,
                            int in_fd,
                            int out_fd,
-                           t_uchar * path);
+                           const t_uchar * path);
 static int ftp_client_pwd (alloc_limits limits,
                            int * errn,
                            int * code,
@@ -229,7 +258,7 @@ struct arch_pfs_vtable ftp_pfs_fns =
 
 
 int
-arch_pfs_ftp_supported_protocol (t_uchar * uri)
+arch_pfs_ftp_supported_protocol (const t_uchar * uri)
 {
   if (!str_cmp_prefix ("ftp://", uri) || !str_cmp_prefix ("wu-ftp://", uri))
     return 1;
@@ -240,7 +269,7 @@ arch_pfs_ftp_supported_protocol (t_uchar * uri)
 
 
 struct arch_pfs_session *
-arch_pfs_ftp_connect (t_uchar * uri)
+arch_pfs_ftp_connect (const t_uchar * uri)
 {
   int errn;
   int code = 0;
@@ -259,9 +288,9 @@ arch_pfs_ftp_connect (t_uchar * uri)
   answer->pfs.vtable = &ftp_pfs_fns;
 
   {
-    t_uchar * prefix;
-    t_uchar * start;
-    t_uchar * stop;
+    const t_uchar * prefix;
+    const t_uchar * start;
+    const t_uchar * stop;
 
     if (!str_cmp_prefix ("ftp://", uri))
       prefix = "ftp://";
@@ -377,13 +406,14 @@ arch_pfs_ftp_connect (t_uchar * uri)
 
 
 static int
-pfs_ftp_file_exists (struct arch_pfs_session * p, t_uchar * path)
+pfs_ftp_file_exists (struct arch_pfs_session * p,
+                     const t_uchar * path)
 {
   struct arch_pfs_ftp_session * arch = (struct arch_pfs_ftp_session *)p;
   t_uchar * abs_path = 0;
   t_uchar * abs_path_dir = 0;
   t_uchar * path_tail = 0;
-  rel_table nlst = 0;
+  rel_table nlst = rel_table_nil;
   int x;
   int answer;
 
@@ -395,7 +425,7 @@ pfs_ftp_file_exists (struct arch_pfs_session * p, t_uchar * path)
   answer = 0;
   for (x = 0; x < rel_n_records (nlst); ++x)
     {
-      if (!str_cmp (path_tail, nlst[x][0]))
+      if (!str_cmp (path_tail, rel_peek_str (nlst, x, 0)))
         {
           answer = 1;
           break;
@@ -409,7 +439,7 @@ pfs_ftp_file_exists (struct arch_pfs_session * p, t_uchar * path)
       nlst = pfs_ftp_directory_files (p, abs_path, 1);
       for (x = 0; x < rel_n_records (nlst); ++x)
         {
-          if (!str_cmp (path_tail, nlst[x][0]))
+          if (!str_cmp (path_tail, rel_peek_str (nlst, x, 0)))
             {
               answer = 1;
               break;
@@ -426,7 +456,8 @@ pfs_ftp_file_exists (struct arch_pfs_session * p, t_uchar * path)
 }
 
 static int
-pfs_ftp_is_dir (struct arch_pfs_session * p, t_uchar * path)
+pfs_ftp_is_dir (struct arch_pfs_session * p,
+                const t_uchar * path)
 {
   struct arch_pfs_ftp_session * arch = (struct arch_pfs_ftp_session *)p;
   int errn;
@@ -457,10 +488,10 @@ pfs_ftp_is_dir (struct arch_pfs_session * p, t_uchar * path)
 }
 
 
-
-
 static t_uchar *
-pfs_ftp_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_ftp_file_contents (struct arch_pfs_session * p,
+                       const t_uchar * path,
+                       int soft_errors)
 {
   int string_fd;
   t_uchar * answer = 0;
@@ -482,7 +513,10 @@ pfs_ftp_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_err
 
 
 static int
-pfs_ftp_get_file (struct arch_pfs_session * p, int file_out_fd, t_uchar * path, int soft_errors)
+pfs_ftp_get_file (struct arch_pfs_session * p,
+                  int file_out_fd,
+                  const t_uchar * path,
+                  int soft_errors)
 {
   struct arch_pfs_ftp_session * arch = (struct arch_pfs_ftp_session *)p;
   int errn;
@@ -565,16 +599,17 @@ pfs_ftp_get_file (struct arch_pfs_session * p, int file_out_fd, t_uchar * path,
 }
 
 
-
 static rel_table
-pfs_ftp_directory_files (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_ftp_directory_files (struct arch_pfs_session * p,
+                         const t_uchar * path,
+                         int soft_errors)
 {
   struct arch_pfs_ftp_session * arch = (struct arch_pfs_ftp_session *)p;
   int errn;
   int code;
   t_uchar * text = 0;
   long text_len;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   t_uchar * nlst_path = 0;
   t_uchar * nlst_path_lastc = 0;
 
@@ -653,18 +688,17 @@ pfs_ftp_directory_files (struct arch_pfs_session * p, t_uchar * path, int soft_e
                   lim_free (0, text);
                   text = 0;
 
-                  raw_list = trim_surrounding_ws (raw_list);
                   answer = rel_ws_split (raw_list);
 
                   lim_free (0, raw_list);
 
                   for (x = 0; x < rel_n_records (answer); ++x)
                     {
-                      t_uchar * t = 0;
+                      t_uchar * t;
 
-                      t = file_name_tail (0, answer[x][0]);
-                      lim_free (0, answer[x][0]);
-                      answer[x][0] = t;
+                      t = file_name_tail (0, rel_peek_str (answer, x, 0));
+                      rel_set_taking (answer, x, 0, rel_make_field_str (t));
+                      lim_free (0, t);
                     }
                 }
             }
@@ -677,9 +711,12 @@ pfs_ftp_directory_files (struct arch_pfs_session * p, t_uchar * path, int soft_e
 }
 
 
-
 static int
-pfs_ftp_put_file (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int file_in_fd, int soft_errors)
+pfs_ftp_put_file (struct arch_pfs_session * p,
+                  const t_uchar * path,
+                  mode_t perms,
+                  int file_in_fd,
+                  int soft_errors)
 {
   struct arch_pfs_ftp_session * arch = (struct arch_pfs_ftp_session *)p;
   int errn;
@@ -762,10 +799,11 @@ pfs_ftp_put_file (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int
 }
 
 
-
-
 static int
-pfs_ftp_mkdir (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int soft_errors)
+pfs_ftp_mkdir (struct arch_pfs_session * p,
+               const t_uchar * path,
+               mode_t perms,
+               int soft_errors)
 {
   struct arch_pfs_ftp_session * arch = (struct arch_pfs_ftp_session *)p;
   int errn;
@@ -793,7 +831,11 @@ pfs_ftp_mkdir (struct arch_pfs_session * p, t_uchar * path, mode_t perms, int so
 
 
 static int
-pfs_ftp_rename (struct arch_pfs_session * p, t_uchar ** errstr, t_uchar * from, t_uchar * to, int soft_errors)
+pfs_ftp_rename (struct arch_pfs_session * p,
+                t_uchar ** errstr,
+                const t_uchar * from,
+                const t_uchar * to,
+                int soft_errors)
 {
   struct arch_pfs_ftp_session * arch = (struct arch_pfs_ftp_session *)p;
   int errn;
@@ -826,7 +868,9 @@ pfs_ftp_rename (struct arch_pfs_session * p, t_uchar ** errstr, t_uchar * from,
 
 
 static int
-pfs_ftp_rmdir (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_ftp_rmdir (struct arch_pfs_session * p,
+               const t_uchar * path,
+               int soft_errors)
 {
   struct arch_pfs_ftp_session * arch = (struct arch_pfs_ftp_session *)p;
   int errn;
@@ -856,7 +900,9 @@ pfs_ftp_rmdir (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
 
 
 static int
-pfs_ftp_rm (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
+pfs_ftp_rm (struct arch_pfs_session * p,
+            const t_uchar * path,
+            int soft_errors)
 {
   struct arch_pfs_ftp_session * arch = (struct arch_pfs_ftp_session *)p;
   int errn;
@@ -954,8 +1000,8 @@ ftp_client_login (alloc_limits limits,
                   long * text_len,
                   int in_fd,
                   int out_fd,
-                  t_uchar * name,
-                  t_uchar * passwd)
+                  const t_uchar * name,
+                  const t_uchar * passwd)
 {
   int user_repl_code;
   t_uchar * user_repl_text;
@@ -1268,7 +1314,7 @@ ftp_client_read_simple_cmd_reply (alloc_limits limit,
 }
 
 static int
-ftp_client_printfmt (int * errn, int fd, char * fmt, ...)
+ftp_client_printfmt (int * errn, int fd, const char * fmt, ...)
 {
   int answer;
   va_list ap;
@@ -1335,7 +1381,9 @@ ftp_client_port (alloc_limits limits,
  *
  */
 static int
-ftp_client_pasv_parse_equal (char *text, t_ulong *h, t_uint16 *p)
+ftp_client_pasv_parse_equal (const char *text,
+                             t_ulong *h,
+                             t_uint16 *p)
 {
     int er;
     t_uchar * start_byte;
@@ -1403,7 +1451,9 @@ ftp_client_pasv_parse_equal (char *text, t_ulong *h, t_uint16 *p)
  *
  */
 static int
-ftp_client_pasv_parse_parenthesis (char *text, t_ulong *h, t_uint16 *p)
+ftp_client_pasv_parse_parenthesis (const char *text,
+                                   t_ulong *h,
+                                   t_uint16 *p)
 {
     int er;
     t_uchar * start_byte;
@@ -1645,7 +1695,7 @@ ftp_client_begin_retr (alloc_limits limits,
                        long * text_len,
                        int in_fd,
                        int out_fd,
-                       t_uchar * path)
+                       const t_uchar * path)
 {
   int c;
 
@@ -1739,7 +1789,7 @@ ftp_client_begin_stor (alloc_limits limits,
                        long * text_len,
                        int in_fd,
                        int out_fd,
-                       t_uchar * path)
+                       const t_uchar * path)
 {
   int c;
 
@@ -1808,7 +1858,7 @@ ftp_client_begin_nlst (alloc_limits limits,
                        int in_fd,
                        int out_fd,
                        int is_wu_ftp,
-                       t_uchar * path,
+                       const t_uchar * path,
                        t_ulong host_addr,
                        t_uint16 port)
 {
@@ -1901,7 +1951,7 @@ ftp_client_mkd (alloc_limits limits,
                 long * text_len,
                 int in_fd,
                 int out_fd,
-                t_uchar * path)
+                const t_uchar * path)
 {
   if (0 > ftp_client_printfmt (errn, out_fd, "MKD %s\r\n", path))
     return -1;
@@ -1924,8 +1974,8 @@ ftp_client_rename (alloc_limits limits,
                    long * text_len,
                    int in_fd,
                    int out_fd,
-                   t_uchar * from,
-                   t_uchar * to)
+                   const t_uchar * from,
+                   const t_uchar * to)
 {
   int rnfr_code;
   t_uchar * rnfr_text;
@@ -2024,7 +2074,7 @@ ftp_client_dele (alloc_limits limits,
                  long * text_len,
                  int in_fd,
                  int out_fd,
-                 t_uchar * path)
+                 const t_uchar * path)
 {
   if (0 > ftp_client_printfmt (errn, out_fd, "DELE %s\r\n", path))
     return -1;
@@ -2047,7 +2097,7 @@ ftp_client_rmd (alloc_limits limits,
                 long * text_len,
                 int in_fd,
                 int out_fd,
-                t_uchar * path)
+                const t_uchar * path)
 {
   if (0 > ftp_client_printfmt (errn, out_fd, "RMD %s\r\n", path))
     return -1;
@@ -2069,7 +2119,7 @@ ftp_client_cwd (alloc_limits limits,
                 long * text_len,
                 int in_fd,
                 int out_fd,
-                t_uchar * path)
+                const t_uchar * path)
 {
   if (0 > ftp_client_printfmt (errn, out_fd, "CWD %s\r\n", path))
     return -1;
