@@ -230,7 +230,7 @@ sftp_write_pkt (struct arch_pfs_sftp_session * p,
   while (*fmt)
     switch (*fmt++)
       {
-        case '8': /* int64 */
+        case '8': /* unsigned long long */
           ll = va_arg (ap, unsigned long long);
           len += sizeof (ll);
           buf = lim_realloc (0, buf, len);
@@ -238,7 +238,7 @@ sftp_write_pkt (struct arch_pfs_sftp_session * p,
           i += sizeof (ll);
           break;
 
-        case '4': /* int32 */
+        case '4': /* t_uint32 */
           l = va_arg (ap, t_uint32);
           len += sizeof (l);
           buf = lim_realloc (0, buf, len);
@@ -246,7 +246,7 @@ sftp_write_pkt (struct arch_pfs_sftp_session * p,
           i += sizeof (l);
           break;
 
-        case '1': /* int8 */
+        case '1': /* int */
           c = va_arg (ap, int);
           len += sizeof (c);
           buf = lim_realloc (0, buf, len);
@@ -254,9 +254,9 @@ sftp_write_pkt (struct arch_pfs_sftp_session * p,
           i += sizeof (c);
           break;
 
-        case 's': /* string  (IE: <int32:length> <string>) */
+        case 's': /* char*,size_t  (IE: <int32:length> <string>) */
           s = va_arg (ap, char *);
-          l = va_arg (ap, int);
+          l = va_arg (ap, size_t);
           len += sizeof (l) + l;
           buf = lim_realloc (0, buf, len);
           WRITE_UINT4 (buf + i, l);
@@ -265,7 +265,7 @@ sftp_write_pkt (struct arch_pfs_sftp_session * p,
           i += l;
           break;
 
-        case 'a': /* attributes */
+        case 'a': /* struct sftp_attrs* */
             attrs = va_arg (ap, struct sftp_attrs *);
           sftp_export_attrs (attrs, &s, &s_len);
           len += s_len;
@@ -468,7 +468,8 @@ sftp_client_do_init (struct arch_pfs_sftp_session * p,
   int pkt_len;
   char *pkt;
 
-  sftp_write_pkt (p, soft_errors, "14", SSH_FXP_INIT, SFTP_VERSION, soft_errors);
+  sftp_write_pkt (p, soft_errors, "14", SSH_FXP_INIT, 
+		  (t_uint32)SFTP_VERSION, soft_errors);
   sftp_server_response (p, &pkt, &pkt_len, SSH_FXP_VERSION, soft_errors);
 
   lim_free (0, pkt);
@@ -653,7 +654,8 @@ sftp_get (struct arch_pfs_sftp_session * p,
   attrs.flags = 0;
   ret = sftp_write_pkt (p, soft_errors,
                         "14s4a", SSH_FXP_OPEN, ++p->request_id,
-                        file, str_length (file), SSH_FXF_READ, &attrs);
+                        file, str_length (file), 
+			(t_uint32)SSH_FXF_READ, &attrs);
   lim_free (0, file);
 
   if (0 > ret)
@@ -689,9 +691,9 @@ sftp_get (struct arch_pfs_sftp_session * p,
       for (; ! eof && p->plreqs <= MIN(MAX_PL_REQS, numpkts + 1); p->plreqs++)
 	{
 	  ret = sftp_write_pkt (p, soft_errors, "14s84", SSH_FXP_READ,
-				++p->request_id, handle, handle_len,
+				++p->request_id, handle, (size_t)handle_len,
 				total_read + SFTP_RWSIZE * p->plreqs,
-				SFTP_RWSIZE);
+				(t_uint32)SFTP_RWSIZE);
 	  if (0 > ret)
 	    {
 	      lim_free (0, handle);
@@ -765,7 +767,7 @@ sftp_get (struct arch_pfs_sftp_session * p,
 
   if (0 > sftp_write_pkt (p, soft_errors,
                           "14s", SSH_FXP_CLOSE, ++p->request_id,
-                          handle, handle_len))
+                          handle, (size_t)handle_len))
     return -1;
 
   lim_free (0, handle);
@@ -876,7 +878,7 @@ pfs_directory_files (struct arch_pfs_session * p,
     {
       if (0 >sftp_write_pkt (pfs, soft_errors,
                              "14s", SSH_FXP_READDIR, ++pfs->request_id,
-                             handle, handle_len))
+                             handle, (size_t)handle_len))
         return rel_table_nil;
 
       if (0 > (i = sftp_server_response (pfs, &pkt, &pkt_len, SSH_FXP_NAME,
@@ -915,7 +917,7 @@ pfs_directory_files (struct arch_pfs_session * p,
           if (!dots)
             {
               file = str_save_n (0, pkt+i, len);
-              rel_add_records (&answer, rel_singleton_record_taking (rel_make_field_str (file)), 0);
+              rel_add_records (&answer, rel_singleton_record_taking (rel_make_field_str (file)), rel_record_null);
             }
 
           i += len;
@@ -945,7 +947,7 @@ pfs_directory_files (struct arch_pfs_session * p,
 
   if (0 > sftp_write_pkt (pfs, soft_errors,
                           "14s", SSH_FXP_CLOSE, ++pfs->request_id,
-                          handle, handle_len))
+                          handle, (size_t)handle_len))
     return rel_table_nil;
   lim_free (0, handle);
 
@@ -1057,7 +1059,9 @@ pfs_put_file (struct arch_pfs_session *p,
   ret = sftp_write_pkt (pfs, soft_errors,
                         "14s4a", SSH_FXP_OPEN, ++pfs->request_id, file,
                         str_length (file),
-                        SSH_FXF_WRITE | SSH_FXF_CREAT | SSH_FXF_TRUNC, &attrs);
+                        (t_uint32)(SSH_FXF_WRITE | SSH_FXF_CREAT | 
+				   SSH_FXF_TRUNC),
+			&attrs);
   lim_free (0, file);
 
   if (0 > ret)
@@ -1102,9 +1106,9 @@ pfs_put_file (struct arch_pfs_session *p,
 	    }
 
 	  ret = sftp_write_pkt (pfs, soft_errors, "14s8s", SSH_FXP_WRITE,
-				++pfs->request_id, handle, handle_len,
+				++pfs->request_id, handle, (size_t)handle_len,
 				total_written + pfs->plreqs * sizeof (buf),
-				buf, len);
+				buf, (size_t)len);
 	  if (0 > ret)
 	    {
 	      lim_free (0, handle);
@@ -1144,7 +1148,7 @@ pfs_put_file (struct arch_pfs_session *p,
 
   ret = sftp_write_pkt (pfs, soft_errors,
                         "14s", SSH_FXP_CLOSE, ++pfs->request_id, handle,
-                        handle_len);
+                        (size_t)handle_len);
   lim_free (0, handle);
 
   if (0 > ret)
