@@ -329,6 +329,7 @@ rel_make_record_8_taking (rel_field f0,
   return answer;
 }
 
+const rel_record rel_record_null;
 
 void
 rel_add_records (rel_table * table, ...)
@@ -467,7 +468,7 @@ rel_ws_split (const t_uchar * string)
       while (*end && !char_is_space (*end))
         ++end;
 
-      rel_add_records (&answer, rel_singleton_record_taking (rel_make_field_str_n (start, end - start)), 0);
+      rel_add_records (&answer, rel_singleton_record_taking (rel_make_field_str_n (start, end - start)), rel_record_null);
 
       start = end;
     }
@@ -495,7 +496,7 @@ rel_nl_split (const t_uchar * string)
       while (*end && (*end != '\n'))
         ++end;
 
-      rel_add_records (&answer, rel_singleton_record_taking (rel_make_field_str_n (start, end - start)), 0);
+      rel_add_records (&answer, rel_singleton_record_taking (rel_make_field_str_n (start, end - start)), rel_record_null);
 
       if (*end)
         start = end + 1;
@@ -837,7 +838,7 @@ rel_cut (rel_cut_spec fields, rel_table t)
   lim = ar_size ((void *)t._r, 0, sizeof (rel_record));
   for (x = 0; x < lim; ++x)
     {
-      rel_add_records (&answer, rel_cut_record (fields, t._r[x]), 0);
+      rel_add_records (&answer, rel_cut_record (fields, t._r[x]), rel_record_null);
     }
 
   return answer;
