@@ -13,6 +13,7 @@
 #include "hackerlab/char/str.h"
 #include "hackerlab/vu/safe.h"
 #include "tla/libarch/cmds.h"
+#include "tla/revision"
 
 
 
@@ -56,8 +57,12 @@ main (int argc, char * argv[])
   if (!str_cmp (cmd, "-V") || !str_cmp (cmd, "--version"))
     {
       safe_printfmt (1, cfg__std__package "\n");
+#ifdef TLA_REVISION
+      safe_printfmt(1, "%s\n", TLA_REVISION);
+#endif
+      safe_printfmt (1, "built on %s\n", __DATE__);
       safe_printfmt (1, "\n");
-      safe_printfmt (1, "Copyright 2003, 2004, 2005 Free Software Foundation, Inc.\n");
+      safe_printfmt (1, "Copyright 2003, 2004, 2005 Free Software Foundation, Inc. and contributors\n");
       safe_printfmt (1, "\n");
       safe_printfmt (1, "This is free software; see the source for copying conditions.\n");
       safe_printfmt (1, "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n");
